from ella import newman

from ella_category_subdomain.models import CategorySubdomain

class CategorySubdomainAdmin(newman.NewmanModelAdmin):
    pass

newman.site.register(CategorySubdomain, CategorySubdomainAdmin)



from urlparse import urlparse, urlunparse

import django.core.urlresolvers as urlresolvers

from ella_category_subdomain.util import get_domain_for_category

def update_parsed_url_list(parsed_url_list):
    # If parsed_url has scheme attribute then preserve it,
    # else default to http.
    parsed_url_list[0] = parsed_url_list[0] or 'http'

def get_url_with_subdomain(parsed_url, category_subdomain):
    # JS: myslim, ze doporuceni je psat mala pismena a bez tecky na konci, pokud jde o jednu vetu
    # We will need mutable version of parsed_url.
    parsed_url_list = list(parsed_url)

    category_path = category_subdomain.category.tree_path

    # Strip category tree path (plus slash at the beginning).
    new_path = parsed_url.path[len(category_path)+1:]

    # Get subdomain for given category.
    domain = category_subdomain.get_subdomain()

    # Change parsed_url_list.
    parsed_url_list[1:3] = domain, new_path

    update_parsed_url_list(parsed_url_list)

    # Construct and return new url.
    return urlunparse(parsed_url_list)

def get_url_without_subdomain(parsed_url):
    parts = parsed_url.netloc.split('.')

    # We will need mutable version of parsed_url.
    parsed_url_list = list(parsed_url)

    parsed_url_list[1] = get_domain_for_category(strip_www = False)
    update_parsed_url_list(parsed_url_list)

    # Construct and return new url.
    return urlunparse(parsed_url_list)

def get_url(url):
    from ella_category_subdomain.models import CategorySubdomain

    # parse url
    parsed_url = urlparse(url)

    # get non-empty URL path items
    path_items = [item for item in parsed_url.path.split('/') if (len(item) > 0)]
    # if the path is not a root one check if the first item of path does not match category
    # subdomain and change the URL accordingly
    if (len(path_items) > 0):
        # get the first path item
        first_path_item = path_items[0]
        # search for the particular subdomain category
        category_subdomain_list = CategorySubdomain.objects.filter(category__slug = first_path_item)
        # change the URL if found
        if (len(category_subdomain_list) == 1):
            url = get_url_with_subdomain(parsed_url, category_subdomain_list[0])

            return url

    # get the URL domain parts
    domain_items = parsed_url.netloc.split('.')
    # get the first domain part
    first_domain_item = domain_items[0]
    # search for the particular subdomain category
    category_subdomain_list = CategorySubdomain.objects.filter(subdomain_slug = first_domain_item)

    # the URL already modified if exists
    if (len(category_subdomain_list) > 0):
        return url
    # fill in the default subdomain otherwise
    else:
        url = get_url_without_subdomain(parsed_url)
        return url

def patch_reverse(reverse):
    def wrapper(*args, **kwargs):
        # Get url from a result of original Django reverse.
        return get_url(reverse(*args, **kwargs))
    wrapper._original_reverse = reverse
    return wrapper




import importlib
import inspect

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models.signals import class_prepared
from django.dispatch import receiver
from django.utils.translation import ugettext_lazy as _
import django.core.urlresolvers as urlresolvers

from ella.core.models import Category

from ella_category_subdomain.monkeypatch import patch_reverse
from ella_category_subdomain.util import get_domain_for_category

class CategorySubdomainManager(models.Manager):

    def get_for_path(self, path):
        """Returns a CategorySubdomain instance for a first part of the path if present in
        the database.
        """
        path_items = [path_item for path_item in path.split('/') if len(path_item)>0]
        result = None
        if (len(path_items) > 0):
            slug = path_items[0]
            try:
                result = self.get(category__slug=slug, category__site__id=settings.SITE_ID)
            except self.model.DoesNotExist:
                pass

        return result

    def get_for_host(self, host):
        """Searches for a CategorySubdomain instance matching the first part of the domain.
        """
        # split the domain into parts
        domain_parts = host.split('.')
        # prepare the default response
        result = None
        # process the domain if it is of the 3rd level or more
        if (len(domain_parts) > 2):
            # get the first part of the domain
            subdomain = domain_parts[0].lower()

            try:
                # get the category subdomain
                result = self.get(subdomain_slug=subdomain, category__site__id=settings.SITE_ID)

            except self.model.DoesNotExist:
                # category subdomain does not exists
                pass

        return result


class CategorySubdomain(models.Model):
    objects = CategorySubdomainManager()

    category = models.OneToOneField(Category)
    subdomain_slug = models.SlugField(max_length=64)

    def __unicode__(self):
        return self.subdomain_slug

    def get_domain(self, strip_www = True):
        domain = get_domain_for_category(self.category, strip_www)
        return domain

    def get_subdomain(self):
        return '%s.%s' % (self.subdomain_slug, self.get_domain())

    def get_absolute_url(self):
        return "http://%s.%s/" % (self.subdomain_slug, self.get_domain(),)

    def clean(self):
        """Validates that only first level category is referenced by the CategorySubdomain
        """
        if ((self.category.tree_parent is None) or
            (self.category.tree_parent.tree_parent is not None)):
            raise ValidationError(_('Subdomain can only reference a first level categories'))

    class Meta:
        unique_together = (('category', 'subdomain_slug'),)
        verbose_name = _('Category Subdomain')
        verbose_name_plural = _('Category Subdomains')

# Flag which indicates that the patches were already implemented
PATCHED = 0

@receiver(class_prepared)
def patch_stuff(sender, **kwargs):
    """The method applies patches upon receiving 'class_prepared' signal. It wraps 'resolve' method of
    the django urlresolvers module and 'get_absolute_url' method of all ella model instances having the
    method.
    """
    # reference the global flag
    global PATCHED
    # bail out it the path has been applied already
    if PATCHED:
        return

    # raise the flag of patch application
    PATCHED = 1

    # prepare the buffer carrying the model class already patched. Each import can bring models already pathed.
    patched_models = {}

    # go through all the installed applications
    for app in settings.INSTALLED_APPS:
        # process only apps belonging to the ella framework and skip self
        if ((app.startswith('ella')) and
            (app != 'ella_category_subdomain')):

            # import the models package
            module = importlib.import_module('.models', app)
            # get all the package members
            module_members = inspect.getmembers(module)

            # wrap all the 'get_absolute_url' method in all the Model subclasses found
            for name, member in module_members:
                if ((inspect.isclass(member)) and
                    (issubclass(member, models.Model)) and
                    (patched_models.get(name) is None)):

                    # record the class has been processed
                    patched_models[name] = 1
                    # wrap the 'get_absolute_url' method
                    if hasattr(member, 'get_absolute_url'):
                        member.get_absolute_url = patch_reverse(member.get_absolute_url)

    # patch the reverse function
    urlresolvers.reverse = patch_reverse(urlresolvers.reverse)



from django.contrib.sites.models import Site
from django.conf import settings

def get_domain_for_category(category=None, strip_www=False):
    """Return site domain with development server port (if DEBUG)."""

    if category is not None:
        domain = category.site.domain
    # get the site domain otherwise
    else:
        domain = Site.objects.get(pk=settings.SITE_ID).domain

    # append the port to the domain when neccessary
    if settings.DEBUG and hasattr(settings, 'DEVELOPMENT_SERVER_PORT'):
        domain = '%s:%s' % (domain, settings.DEVELOPMENT_SERVER_PORT,)

    www = 'www.'
    domain = domain[len(www):] if domain.startswith(www) and strip_www else domain
    return domain




import logging

from urlparse import urlparse, urlunparse

from django.conf import settings
from django.http import HttpResponseRedirect, Http404

from ella_category_subdomain.conf import ella_category_subdomain_settings
from ella_category_subdomain.models import CategorySubdomain
from ella_category_subdomain.util import get_domain_for_category

class CategorySubdomainMiddleware(object):
    """The middleware is requirement for the correct function of the application.
    It rewrites the request path to the original state for the normal django processing.
    """

    def __init__(self):
        super(CategorySubdomainMiddleware, self).__init__()
        self.static_prefixes = [settings.MEDIA_URL,]
        self.log = logging.getLogger("%s.%s" % (__name__, self.__class__.__name__))

    def process_request(self, request):
        host = request.get_host()
        self.log.debug("Path: %s", request.path)
        self.log.debug("Host: %s", host)
        path_subdomain = CategorySubdomain.objects.get_for_path(request.path)
        self.log.debug("Path subdomain: %s" % (path_subdomain))
        if ((path_subdomain is not None) and
            (not ella_category_subdomain_settings.OLD_STYLE_URL)):
            raise Http404

        self.category_subdomain = CategorySubdomain.objects.get_for_host(host)

        return None

    def process_view(self, request, view_func, view_args, view_kwargs):
        if (self.category_subdomain is not None):
            self.log.debug("process view: %s, %s, %s", view_func, view_args, view_kwargs)

            for prefix in self.static_prefixes:
                if request.path_info.startswith(prefix):
                    return

            view_kwargs['category'] = self.category_subdomain.category.tree_path
            self.log.debug("process view modified: %s, %s, %s", view_func, view_args, view_kwargs)

        return None


class CategorySubdomainRedirectMiddleware(object):
    """The middleware allows to use original URL for the categories which are
    configured for subdomains. It redirect the old style URL of the subdomain
    category to the correct subdomain.
    """

    def __init__(self):
        self.log = logging.getLogger("%s.%s" % (__name__, self.__class__.__name__))


    def process_request(self, request):
        redirect = self._get_redirect_if_old_url(request)
        return redirect

    def _get_redirect_if_old_url(self, request):
        """Returns a HttpResponseRedirect instance when the category configured for a
        subdomain is detected in the path (i.e. as if there was no subdomain) and redirects
        the request to correct subdomain URL.
        """
        # get the site domain
        domain = get_domain_for_category(category=None, strip_www=False)
        # get the current request host
        host = request.get_host()

        self.log.debug("Host: %s, domain: %s", host, domain)

        # try to find a subdomain for a first category in the path if they match
        if host == domain:
            # search for a category subdomain matching the first path of the domain
            category_subdomain = CategorySubdomain.objects.get_for_path(request.path)
            #
            if category_subdomain is not None:
                # get the absolute uri of the request
                request_absolute_uri = request.build_absolute_uri()
                # parse the uri
                parsed_url_list = list(urlparse(request_absolute_uri))
                # get domain name for the category subdomain
                new_domain = category_subdomain.get_subdomain()
                # cut off the first category part of the path
                new_path = request.path[len(category_subdomain.category.tree_path)+1:]
                # replace domain and path in uri
                parsed_url_list[1:3] = new_domain, new_path
                # redirect to the new uri
                return HttpResponseRedirect(urlunparse(parsed_url_list))




from setuptools import setup
from ella_category_subdomain import __versionstr__
import sys

setup(
    name = 'Ella-Category-Subdomain',
    version = __versionstr__,
    description = 'Use Ella categories as subdomains.',
    long_description = """Use selected Ella categories as subdomains without
    the need to create many sites and for easier administration.

    It is usable for SEO, microsites, landing pages and similar things.""",
    author = 'Sanoma Media Praha s.r.o.',
    author_email = 'online-dev@sanomamedia.cz',
    maintainer = 'Vitek Pliska',
    maintainer_email='whit@jizak.cz',
    license = 'BSD',
    url='http://github.com/sanomacz/ella-category-subdomain/',

    packages = ('ella_category_subdomain',),
    include_package_data = True,

    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Programming Language :: Python :: 2.5",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Operating System :: OS Independent",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],

    install_requires = [
        'setuptools>=0.6b1',
        'ella>=2.0',
        'importlib'
    ] if sys.version < '2.7'
    else [
        'setuptools>=0.6b1',
        'ella>=2.0',
    ],
)

